/*
 * $Id: rc.c,v 1.2 2006-10-19 15:28:04 andreas Exp $
 */

/*
 * Copyright (c) 2006 Andreas Schroeder <andreas@a-netz.de>
 *
 * Implement functions of an infrared remote control and RC5 protocol with an
 * Atmel ATTiny 2313.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
 
 // Angepasst auf ATmega8 mit 16 Mhz Quarz.
 
 // Test, 2 Startbit, 8 Datenbit

#include <avr/io.h>
#include <avr/interrupt.h>
//#include <avr/sleep.h>
#include "rc5codes.h"

/**
 * Device code to send
 */
//#define RC5_DEVICE_CODE       RC5_DEV_EXPERIMENTAL

#define RC5_DEVICE_CODE       0x00

/**
 * Maximum bit count before transmission is finished. Minimum (as
 * protocol standard says) is 64. Higher values yield a lower repetition rate.
 */
#define MAX_BITCOUNT 128

//                   0x3000 = 11000000000000
//#define RC5_TEMPLATE (0x3000 | (RC5_DEVICE_CODE << 6))
//                   0x300 = 1100000000 nicht schieben, da 8 bit Daten
#define RC5_TEMPLATE (0x300 | (RC5_DEVICE_CODE))

volatile uint16_t rc5data = 0;

volatile uint8_t bitcounter = MAX_BITCOUNT;
volatile uint8_t halfbitcounter;

uint8_t led = 0;

uint8_t cmd = 0;

// modulation square wave generation
// compare output mode: toggle OC0A on compare match
// if COM0Ax are *all* unset, the waveform generation is disabled and
// the normal PORTx value is seen on the OC0A pin.
//#define ENABLE_MODULATION (TCCR0A |= _BV(COM0A0))
//#define DISABLE_MODULATION (TCCR0A &= ~_BV(COM0A0))

//ATmega8 Timer2
#define ENABLE_MODULATION (TCCR2 |= _BV(COM20))
#define DISABLE_MODULATION (TCCR2 &= ~_BV(COM20))

// activate half bit timer
#define ENABLE_BIT_TIMER (TCCR1B |= _BV(CS10))
#define DISABLE_BIT_TIMER (TCCR1B &= ~_BV(CS10))

// button control
// MCUCR is setup by default to trigger INT0 on low level, so only external
// interrupt needs to be turned on or off
//#define ENABLE_BUTTONS (GIMSK |= _BV(INT0))
//#define DISABLE_BUTTONS (GIMSK &= ~_BV(INT0))


volatile uint8_t Pulsdauer = 0;
volatile uint32_t i = 0;

void (steigende_Flanke)(void);
void (fallende_Flanke)(void);

SIGNAL(SIG_INTERRUPT0)
{
	if( MCUCR == 0x03) // Testen auf steigende Flanke
	{	
	steigende_Flanke();
	return;
	}
	
	if( MCUCR == 0x02 ) // Testen auf fallende Flanke
	{
	fallende_Flanke();
	return;
	}	
}
void (steigende_Flanke)(void)
{
	TCNT0 = 0; // Timer Reset
	TCCR0 = _BV (CS02);  // Timer2 1/256 MCU start
	MCUCR = 0x02; // INT0 fallende Flanke
}
void (fallende_Flanke)(void)
{
	TCCR0 = 0x00; // Timer2 Stoppen
	Pulsdauer = TCNT0; // Timer Wert = Pulsdauer
	MCUCR = 0x03; // INT0 auf steigende Flanke
}
/*
ISR(INT0_vect) {
  uint8_t buttons = (~PIND) & (_BV(PD0) | _BV(PD1) | _BV(PD3) | _BV(PD4) | _BV(PD5));
  uint8_t cmd = 0;

  switch(buttons) {
  case _BV(PD0):
    //cmd = RC5_CMD_VOLUME_INC;
	cmd = RC5_CMD_0;
    break;
  case _BV(PD1):
    //cmd = RC5_CMD_VOLUME_DEC;
	cmd = RC5_CMD_1;
    break;
  case _BV(PD3):
    //cmd = RC5_CMD_PROGRAM_INC;
	cmd = RC5_CMD_2;
    break;
  case _BV(PD4):
    //cmd = RC5_CMD_PROGRAM_DEC;
	cmd = RC5_CMD_3;
    break;
  case _BV(PD5):
    //cmd = RC5_CMD_STANDBY;
	cmd = RC5_CMD_4;
    break;
  }

  // assemble rc5 packet from TEMPLATE (2 start bits and device identifier)
  // and 6 command bits
  rc5data = RC5_TEMPLATE | (cmd & 0x3f);

  // activate timer 1 (half bit timer) to write out bitstream
  bitcounter = 0;
  halfbitcounter = 0;
  sleep_disable();
  DISABLE_BUTTONS;
  ENABLE_BIT_TIMER;
}
*/

ISR(TIMER1_COMPA_vect) {
  // 0x2000 = 10000000000000
  //uint8_t bit = ((rc5data & 0x2000) >> 13);
  uint8_t bit = ((rc5data & 0x200) >> 9);
  uint8_t half = (bit ^ ((halfbitcounter == 0) ? 1 : 0));

  // transmit 14 bit rc5 packet
  //if(half && bitcounter < 14) {
  if(half && bitcounter < 10) {
    ENABLE_MODULATION;
	// LED an PB4 an
	//PORTB &= ~_BV(PB4);
	
	//LEDs an
	PORTB = led;
	
	//LEDs aus
	//PORTB |= _BV(PB2) | _BV(PB1) | _BV(PB0);


  } else {
    DISABLE_MODULATION;
	// LED an PB4 aus
	//PORTB |= _BV(PB4);
	
	//LEDs aus
	//PORTB |= _BV(PB2) | _BV(PB1) | _BV(PB0);
	
	//LEDs an
	//PORTB = led;
	
	PORTB = 0;
	
  }

  if(bitcounter >= MAX_BITCOUNT) {
    // we can now prepare for the next key press
    //ENABLE_BUTTONS;
    DISABLE_BIT_TIMER;
    //sleep_enable();
  //} else if(bitcounter == 14) {
  } else if(bitcounter == 10) {
    // rc5 packet has been transmitted
    DISABLE_MODULATION;
  }

  if(halfbitcounter == 0) {
    halfbitcounter++;
  } else {
    halfbitcounter = 0;
    bitcounter++;
    rc5data <<= 1; // shift left rc5 data
  }
}

int main(void) {
  // setup rc oscillator clock divider to div 1
  // (=> 8MHz with internal RC oscollator)
  //CLKPR = _BV(CLKPCE); // enable clock prescale change
  //CLKPR = 0; // div1 => 8 MHz

  // setup waveform generation to 36kHz on timer 0
  //TCCR0A |= _BV(WGM01); // Clear timer on compare match
  //TCCR0B |= _BV(CS00); // timer source is system clock
  
  //ATmega8 Timer2
  TCCR2 |= _BV(WGM21); // Clear timer on compare match
  TCCR2 |= _BV(CS00); // timer source is system clock
  //OCR0A = 107; // 36kHz @ 8 (7,77) MHz
  // verify correct frequency! The internal oscillator is not really accurate
  //OCR0A = 105;
  
  //ATmega8 Timer2
  //angepasst fr Internen Oszilator 8 Mhz
  //OCR2 = 109;
  
  //ATmega8 Timer2
  //angepasst fr 8 Mhz Quarz
  //OCR2 = 107; (mu 0x6F (111))
  //OCR2 = 109;
  
  //ATmega8 Timer2
  //angepasst fr 16 Mhz Quarz
  OCR2 = 222;  //(mu 0xDE (222)

  //OCR0A = 27; // 36kHz @ 1 MHz

  //DDRB |= _BV(PB2); // port b.2 as output
  //PORTB &= ~_BV(PB2); // pb2 is lo
  
  //ATmega8 PB3 OC2
  DDRB |= _BV(PB3) | _BV(PB2) | _BV(PB1) | _BV(PB0); // port pb3 pb2 as output
  //PORTB &= ~_BV(PB3);// | ~_BV(PB2); // pb3 pb2 is lo
  PORTB |= ~_BV(PB3) | ~_BV(PB2) | ~_BV(PB1) | ~_BV(PB0); // pb3 pb2 is lo

  // setup half-bit clock (555Hz) on timer1
  TCCR1A = 0;
  TCCR1B = _BV(WGM12); // Clear timer on compare match
  // verify correct frequency! The internal oscillator is not really accurate
  //OCR1A = 7049; // ~ 889us @ 8MHz  (mu 0x1BC8 , ist 0x1B89 )
  OCR1A = 14224; // ~ 889us @ 16MHz  (mu 0x3790 , )

  TIMSK |= _BV(OCIE1A); // output compare interrupt enable 1a

  // button port is input, pullup enabled
  // buttons on PD0,1,3,4,5, int0 on pd2
  //PORTD = _BV(PD0) | _BV(PD1) | _BV(PD2) | _BV(PD3) | _BV(PD4) | _BV(PD5);
  
  //PD2 Int0 fr RC Signal
  PORTD = 0x04;       // Pull up on PD2 
  DDRD = 0xFB;        // PD2 in Rest out

  // save energy, disable analog comaprator
  ACSR |= _BV(ACD);

  // setup external interrupt
  //ENABLE_BUTTONS;
  
  /* Int0 steigende Flanke / INT0 enable */
  MCUCR = _BV (ISC01) | _BV (ISC00);
  GICR = _BV (INT0);

  sei();
  // set sleep mode to power down mode (i.e. all functions except external
  // level (!) interrupts are disabled. Tiny2313 needs ~ 0.5uA then.
  //set_sleep_mode( _BV(SM0) | _BV(SM1) );

  // enable sleep mode
  //sleep_enable();

  for(;;) {
    // sleep and wait for external interrupt (from button press)
    //sleep_cpu();
	
	//uint8_t cmd = 0;
	
	//if (Pulsdauer < 34) // Anlage aus, Demo aller Funktionen
	if (Pulsdauer < 68) // Anlage aus, Demo aller Funktionen
		{
	//	cmd = RC5_CMD_0;
		//led = 0;
		//alle LEDs
		led = 7;
		// LEDs aus
		//PORTB &= ~ _BV(PB0);
		//PORTB &= ~ _BV(PB1);
		//PORTB &= ~ _BV(PB2);
		//PORTB &= ~ _BV(PB4);
		
		// LEDs an
		//PORTB |= _BV(PB0);
		//PORTB |= _BV(PB1);
		//PORTB |= _BV(PB2);
		//PORTB |= _BV(PB4);
		
		}
	// Pulsdauer Servolow
	//else if (Pulsdauer > 34 && Pulsdauer < 46)
	else if (Pulsdauer >= 68 && Pulsdauer < 85)
		{
		//cmd = RC5_CMD_0;
		
		// Test RGB LED rot
	//	cmd = RC5_CMD_1;
		led = 1;
		// LEDs aus
		//PORTB &= ~ _BV(PB0);
		//PORTB &= ~ _BV(PB1);
		//PORTB &= ~ _BV(PB2);
		//PORTB &= ~ _BV(PB4);
		
		// LEDs an
		//PORTB |= _BV(PB0);
		//PORTB |= _BV(PB1);
		//PORTB |= _BV(PB2);
		//PORTB |= _BV(PB4);
		}
	// Pulsdauer Servomitte
	//else if (Pulsdauer > 45 && Pulsdauer < 51)
	else if (Pulsdauer >= 85 && Pulsdauer < 110)
		{
		//cmd = RC5_CMD_1;
		
		// Test RGB LED grn
	//	cmd = RC5_CMD_2;
		led = 2;
		// LEDs aus
		//PORTB &= ~ _BV(PB0);
		//PORTB &= ~ _BV(PB1);
		//PORTB &= ~ _BV(PB2);
		//PORTB &= ~ _BV(PB4);
		
		// LEDs an
		//PORTB |= _BV(PB0);
		//PORTB |= _BV(PB1);
		//PORTB |= _BV(PB2);
		//PORTB |= _BV(PB4);
		}
	// Pulsdauer Servohigh	
	//else if (Pulsdauer > 50 && Pulsdauer < 75)
	else if (Pulsdauer >= 110 && Pulsdauer < 150)
		{
		//cmd = RC5_CMD_2;
		
		// Test RGB LED blau
		//cmd = RC5_CMD_4;
	//	cmd = RC5_CMD_3;
		led = 4;
		// LEDs aus
		//PORTB &= ~ _BV(PB0);
		//PORTB &= ~ _BV(PB1);
		//PORTB &= ~ _BV(PB2);
		//PORTB &= ~ _BV(PB4);
		
		// LEDs an
		//PORTB |= _BV(PB0);
		//PORTB |= _BV(PB1);
		//PORTB |= _BV(PB2);
		//PORTB |= _BV(PB4);
		}

  // zum testen, Datenbit erhhen
  cmd ++;
	
  // assemble rc5 packet from TEMPLATE (2 start bits and device identifier)
  // and 6 command bits
  // 0x3F = 111111
  //rc5data = RC5_TEMPLATE | (cmd & 0x3f);
  // 
  rc5data = RC5_TEMPLATE | (cmd);

  // activate timer 1 (half bit timer) to write out bitstream
  bitcounter = 0;
  halfbitcounter = 0;
  //sleep_disable();
  //DISABLE_BUTTONS;
  ENABLE_BIT_TIMER;
  
  //for(i=0;i<200000;i++);
  for(i=0;i<100000;i++);
  
 
	
  }
}

